/***************************************************************************

    M.A.M.E.32  -  Multiple Arcade Machine Emulator for WinCE
    Win32 Portions Copyright (C) 1997-98 Michael Soderstrom and Chris Kirmse
    
    This file is part of MAME32, and may only be used, modified and
    distributed under the terms of the MAME license, in "readme.txt".
    By continuing to use, modify or distribute this file you indicate
    that you have read the license and understand and accept it fully.

 ***************************************************************************/

/***************************************************************************

  Keyboard.c

 ***************************************************************************/
#include "inptport.h"
#include "ZKeyboard.h"

#include "HostPalm.h"
//#include <TwChars.h>

#include "Playgame.h"
#include "GDIDisplay.h"


//#define NUMKEYSTATES    256


static RectangleType srcRectA,srcRectB,srcRectC,srcRectD,srcRectE;
UInt32 CurrentKey;
Int16 pScreenX = 0;
Int16 pScreenY = 0;
Boolean pPenDown = false;

/***************************************************************************
    function prototypes
 ***************************************************************************/

int          Keyboard_init(void);
static void         Keyboard_exit(void);

static const struct KeyboardInfo * Keyboard_get_key_list(void);
static void         Keyboard_customize_inputport_defaults(struct ipd *defaults);
static int          Keyboard_is_key_pressed(int keycode);
static int          Keyboard_wait_keypress(void);

//static BOOL         Keyboard_OnMessage(HWND hWnd, UINT Msg, WPARAM wParam, LPARAM lParam, LRESULT* pResult);
static void         Keyboard_AdjustKeylist(void);

/***************************************************************************
    External variables
 ***************************************************************************/

struct OSDKeyboard  Keyboard = 
{
	{ Keyboard_init },                         /* init                         */
    { Keyboard_exit },                         /* exit                         */
    { Keyboard_get_key_list },                 /* get_key_list                 */
    { Keyboard_customize_inputport_defaults }, /* customize_inputport_defaults */
    { Keyboard_is_key_pressed },               /* is_key_pressed               */
    { Keyboard_wait_keypress }                /* wait_keypress                */
    //{ Keyboard_OnMessage }                     /* OnMessage                    */
};

/***************************************************************************
    Internal structures
 ***************************************************************************/

struct tKeyboard_private
{
    BOOL m_bPauseKeyPressed;
    BOOL m_key_pressed;
    int  m_DefaultInput;
};

/***************************************************************************
    Internal variables
 ***************************************************************************/

static struct tKeyboard_private This;

/***************************************************************************
    External OSD functions  
 ***************************************************************************/

/*
    put here anything you need to do when the program is started. Return 0 if 
    initialization was successful, nonzero otherwise.
*/
int Keyboard_init(void)
{
    //int     i;
    //char    KeyState[NUMKEYSTATES];
    int width_div_3, height_div_3;
    
    width_div_3 = (screenWidth-menuWidth)/3;
    height_div_3 = (screenHeight-menuHeight)/3;

    memset(&This, 0, sizeof(struct tKeyboard_private));

	This.m_DefaultInput = INPUT_LAYOUT_HR;

	/*MakeRect(srcRectA,0,0,180,120);
	MakeRect(srcRectB,300,0,180,120);
	MakeRect(srcRectC,0,200,180,120);
	MakeRect(srcRectD,300,200,180,120);
	MakeRect(srcRectE,180,120,120,80);*/

	RctSetRectangle(&srcRectA,0,0,width_div_3,height_div_3);
	RctSetRectangle(&srcRectB,width_div_3 + width_div_3,0,width_div_3,height_div_3);
	RctSetRectangle(&srcRectC,width_div_3,height_div_3,width_div_3,height_div_3);
	RctSetRectangle(&srcRectD,0,height_div_3 + height_div_3,width_div_3,height_div_3);
	RctSetRectangle(&srcRectE,width_div_3 + width_div_3,height_div_3 + height_div_3,width_div_3,height_div_3);


    /* Clear keyboard state. */
//    GetKeyboardState(KeyState);

   //for (i = 0; i < NUMKEYSTATES; i++)
     //   KeyState[i] &= 0x01;

//    SetKeyboardState(KeyState);


    return 0;
}

/*
    put here cleanup routines to be executed when the program is terminated.
*/
void Keyboard_exit(void)
{
}

// Keyboard Definitions
static struct KeyboardInfo keylist[] =
{
    { "A",          'A',                KEYCODE_A },
    { "B",          'B',                KEYCODE_B },
    { "C",          'C',                KEYCODE_C },
    { "D",          'D',                KEYCODE_D },
    { "E",          'E',                KEYCODE_E },
    { "F",          'F',                KEYCODE_F },
    { "G",          'G',                KEYCODE_G },
    { "H",          'H',                KEYCODE_H },
    { "I",          'I',                KEYCODE_I },
    { "J",          'J',                KEYCODE_J },
    { "K",          'K',                KEYCODE_K },
    { "L",          'L',                KEYCODE_L },
    { "M",          'M',                KEYCODE_M },
    { "N",          'N',                KEYCODE_N },
    { "O",          'O',                KEYCODE_O },
    { "P",          'P',                KEYCODE_P },
    { "Q",          'Q',                KEYCODE_Q },
    { "R",          'R',                KEYCODE_R },
    { "S",          'S',                KEYCODE_S },
    { "T",          'T',                KEYCODE_T },
    { "U",          'U',                KEYCODE_U },
    { "V",          'V',                KEYCODE_V },
    { "W",          'W',                KEYCODE_W },
    { "X",          'X',                KEYCODE_X },
    { "Y",          'Y',                KEYCODE_Y },
    { "Z",          'Z',                KEYCODE_Z },
    { "0",          '0',                KEYCODE_0 },
    { "1",          '1',		        KEYCODE_1 },
    { "2",          '2',		        KEYCODE_2 },
    { "3",          '3',		        KEYCODE_3 },
    { "4",          '4',                KEYCODE_4 },
    { "5",          '5',                KEYCODE_5 },
    { "6",          '6',                KEYCODE_6 },
    { "7",          '7',                KEYCODE_7 },
    { "8",          '8',                KEYCODE_8 },
    { "9",          '9',                KEYCODE_9 },
    { "ESC",        vchrLaunch,          KEYCODE_ESC },
    //{ "~",          0xC0,               KEYCODE_TILDE },
    //{ "-",          0xBD,               KEYCODE_MINUS },
    //{ "=",          0xBB,               KEYCODE_EQUALS },
    //{ "BKSPACE",    VK_BACK,            KEYCODE_BACKSPACE },
    { "TAB",        	chrTab,             KEYCODE_TAB },
    //{ "[",          0xDB,               KEYCODE_OPENBRACE },
    //{ "]",          0xDD,               KEYCODE_CLOSEBRACE },
    { "ENTER",      vchrRockerCenter,          KEYCODE_ENTER },
    //{ ";",          0xBA,               KEYCODE_COLON },
    //{ "'",          0xDE,               KEYCODE_QUOTE },
    //{ "\\",			0xDC,		        KEYCODE_BACKSLASH },
    //{ ",",          0xBC,               KEYCODE_COMMA },
    //{ ".",          0xBE,               KEYCODE_STOP },
    //{ "/",          0xBF,               KEYCODE_SLASH },
    { "SPACE",      vchrHard3,           KEYCODE_SPACE },
    //{ "INS",        VK_INSERT,          KEYCODE_INSERT },
    //{ "DEL",        VK_DELETE,          KEYCODE_DEL },
    //{ "HOME",       VK_HOME,            KEYCODE_HOME },
    //{ "END",        VK_END,             KEYCODE_END },
    //{ "PGUP",       VK_PRIOR,           KEYCODE_PGUP },
    //{ "PGDN",       VK_NEXT,            KEYCODE_PGDN },
    { "LEFT",       vchrRockerLeft,            KEYCODE_LEFT },
    { "RIGHT",      vchrRockerRight,           KEYCODE_RIGHT },
    { "UP",         vchrRockerUp,              KEYCODE_UP },
    { "DOWN",       vchrRockerDown,            KEYCODE_DOWN },
    //{ "/ PAD",      VK_DIVIDE,          KEYCODE_SLASH_PAD },
    //{ "* PAD",      VK_MULTIPLY,        KEYCODE_ASTERISK },
    //{ "- PAD",      VK_SUBTRACT,        KEYCODE_MINUS_PAD },
    //{ "+ PAD",      VK_ADD,             KEYCODE_PLUS_PAD },
    //{ ". PAD",      VK_DECIMAL,         KEYCODE_DEL_PAD },
    //{ "PRTSCR",     VK_PRINT,           KEYCODE_PRTSCR },
    //{ "PAUSE",      VK_PAUSE,           KEYCODE_PAUSE },
    { "LSHIFT",     vchrHard4,          KEYCODE_LSHIFT },
    //{ "RSHIFT",     VK_RSHIFT,          KEYCODE_RSHIFT },
    { "LCTRL",      vchrHard1,        KEYCODE_LCONTROL },
    //{ "RCTRL",      VK_RCONTROL,        KEYCODE_RCONTROL },
    //{ "LALT",       VK_LMENU,           KEYCODE_RALT },
    //{ "RALT",       VK_RMENU,           KEYCODE_RALT },
    //{ "SHIFT",      VK_SHIFT,           KEYCODE_LSHIFT },
    //{ "CTRL",       VK_CONTROL,         KEYCODE_LCONTROL },
    { "ALT",        vchrHard2,            KEYCODE_LALT },
    //{ "LWIN",       VK_LWIN,            KEYCODE_OTHER },
    //{ "RWIN",       VK_RWIN,            KEYCODE_OTHER },
    //{ "MENU",       VK_APPS,            KEYCODE_OTHER },
    //{ "SCRLOCK",    VK_SCROLL,          KEYCODE_SCRLOCK },
    //{ "NUMLOCK",    VK_NUMLOCK,         KEYCODE_NUMLOCK },
    //{ "CAPSLOCK",   VK_CAPITAL,         KEYCODE_CAPSLOCK },*/
      { "F3",			vchrCalc,			KEYCODE_F3 },
    { 0, 0, 0 } /* end of table */
};

/*
  return a list of all available keys (see input.h)
*/
const struct KeyboardInfo* Keyboard_get_key_list(void)
{
    return keylist;
}

/*
  inptport.c defines some general purpose defaults for key bindings. They may be
  further adjusted by the OS dependant code to better match the available keyboard,
  e.g. one could map pause to the Pause key instead of P, or snapshot to PrtScr
  instead of F12. Of course the user can further change the settings to anything
  he/she likes.
  This function is called on startup, before reading the configuration from disk.
  Scan the list, and change the keys you want.
*/
void Keyboard_customize_inputport_defaults(struct ipd *defaults)
{
    Keyboard_CustomizeInputportDefaults(This.m_DefaultInput, defaults);
}

/*
  tell whether the specified key is pressed or not. keycode is the OS dependant
  code specified in the list returned by osd_customize_inputport_defaults().
*/
static inline bool PalmPollKeyKey(UInt32 button, UInt32 current)
{
	if (button == 0)
		return false;
		
	if ((button & current) == button)
		return true;
	else
		return false;
}

//#define PalmGetKey(button) (((button&KeyCurrentState())==button)?1:0) 
//#define PalmPollKeyKey(button, current) ((((button)&(current))==button)?1:0) 
/*#define dUP 0x00010002
#define dRT 0x00080010
#define dDN 0x00020004
#define dLT 0x00040008
#define dCT 0x00100000
#define bUP 0x10000002
#define bRT 0x02000000
#define bDN 0x40000004
#define bLT 0x01000000*/

//Zodiac defines
#define zJoyUP 0x00010002
#define zJoyRT 0x00080010
#define zJoyDN 0x00020004
#define zJoyLT 0x00040008
#define zJoyCT 0x00100000
#define tLT 0x04000000
#define tRT 0x08000000
#define zActUP 0x10000002
#define zActRT 0x20000040
#define zActDN 0x40000004
#define zActLT 0x80000020
#define bFC 0x00008000
#define bLaunch 0x00800000
#define zPower  0x00000001


int Keyboard_is_key_pressed(int keycode)
{
	  CurrentKey = KeyCurrentState();
	  
	EvtGetPenNative (WinGetDrawWindow(),&pScreenX, &pScreenY, &pPenDown);  //Slows down Zodiac until screen tap???
	
	//AK - most likely event is no key pressed.  Put this here for speed.
	if (!CurrentKey && !pPenDown)
		return 0;
	
		if ((keycode == '5') && ((KeyStruct[ConfigP1Credit].Key) && (PalmPollKeyKey(KeyStruct[ConfigP1Credit].Key,CurrentKey))))
		{
			return keycode;
		}
		else if ((keycode == '5') && MenuOption > 0 && pPenDown && RctPtInRectangle(pScreenX,pScreenY,&StatusBar.RectStat[2]))
		{
				return keycode;	
		}
		else if ((keycode == '1') && ((KeyStruct[ConfigP1Start].Key) && (PalmPollKeyKey(KeyStruct[ConfigP1Start].Key,CurrentKey))))
		{
				return keycode;
		}
		else if((keycode == '1') && MenuOption > 0 &&  pPenDown && RctPtInRectangle(pScreenX,pScreenY,&StatusBar.RectStat[3]))
		{
				return keycode;
		}
		else if ((keycode == vchrLaunch) && (KeyStruct[ConfigExit].Key) &&  (PalmPollKeyKey(KeyStruct[ConfigExit].Key,CurrentKey)))
			return keycode;
		else if ((keycode == vchrLaunch) && MenuOption > 0 && pPenDown && RctPtInRectangle(pScreenX,pScreenY,&StatusBar.RectStat[0]))
			return keycode;
		else if ((keycode == vchrRockerLeft)  && (KeyStruct[ConfigUp].Key) &&  ZRotateEnable == 1 && (PalmPollKeyKey(KeyStruct[ConfigUp].Key,CurrentKey)))
		    return keycode;
		else if ((keycode == vchrRockerLeft)  && (KeyStruct[ConfigLeft].Key) &&  ZRotateEnable == 0 && (PalmPollKeyKey(KeyStruct[ConfigLeft].Key,CurrentKey)))
		    return keycode;
		else if ((keycode == vchrRockerRight) && (KeyStruct[ConfigDown].Key) &&  ZRotateEnable == 1 && (PalmPollKeyKey(KeyStruct[ConfigDown].Key,CurrentKey)))
			  return keycode;
		else if ((keycode == vchrRockerRight) && (KeyStruct[ConfigRight].Key) &&  ZRotateEnable == 0 && (PalmPollKeyKey(KeyStruct[ConfigRight].Key,CurrentKey)))
			  return keycode;
		else if (keycode == vchrRockerUp && (KeyStruct[ConfigRight].Key) &&  ZRotateEnable == 1 && (PalmPollKeyKey(KeyStruct[ConfigRight].Key,CurrentKey)))
			  return keycode;
		else if (keycode == vchrRockerUp && (KeyStruct[ConfigUp].Key) &&  ZRotateEnable == 0 && (PalmPollKeyKey(KeyStruct[ConfigUp].Key,CurrentKey)))
			  return keycode;
		else if (keycode == vchrRockerDown && (KeyStruct[ConfigLeft].Key) &&  ZRotateEnable == 1 && (PalmPollKeyKey(KeyStruct[ConfigLeft].Key,CurrentKey)))
			  return keycode;
		else if (keycode == vchrRockerDown && (KeyStruct[ConfigDown].Key) &&  ZRotateEnable == 0 && (PalmPollKeyKey(KeyStruct[ConfigDown].Key,CurrentKey)))
			  return keycode;
		else if (keycode == vchrHard1 && (KeyStruct[ConfigButton1].Key) &&  (PalmPollKeyKey(KeyStruct[ConfigButton1].Key,CurrentKey)))
			  return keycode;
		else if (keycode == vchrHard2 && (KeyStruct[ConfigButton2].Key) &&  (PalmPollKeyKey(KeyStruct[ConfigButton2].Key,CurrentKey)))
			  return keycode;
		else if (keycode == vchrHard3 && (KeyStruct[ConfigButton3].Key) &&  (PalmPollKeyKey(KeyStruct[ConfigButton3].Key,CurrentKey)))
			  return keycode;
		else if (keycode == vchrHard4 && (KeyStruct[ConfigButton4].Key) &&  (PalmPollKeyKey(KeyStruct[ConfigButton4].Key,CurrentKey)))
			  return keycode;
		else if (keycode == 'P' && (KeyStruct[ConfigPause].Key) &&  (PalmPollKeyKey(KeyStruct[ConfigPause].Key,CurrentKey)))
			  return keycode;
		else if ((keycode == 'P') && MenuOption > 0 && pPenDown && RctPtInRectangle(pScreenX,pScreenY,&StatusBar.RectStat[8]))
			return keycode;
		else if (keycode == vchrRockerCenter && (KeyStruct[ConfigMenuSelect].Key) &&  (PalmPollKeyKey(KeyStruct[ConfigMenuSelect].Key,CurrentKey)))
			  return keycode;
		else if ((keycode == vchrRockerCenter) && pPenDown && RctPtInRectangle(pScreenX,pScreenY,&StatusBar.RectStat[5]))
			return keycode;
		else if (keycode == chrTab && (KeyStruct[ConfigMenu].Key) &&  (PalmPollKeyKey(KeyStruct[ConfigMenu].Key,CurrentKey)))
			  return keycode;
		else if ((keycode == chrTab) && MenuOption > 0 && pPenDown && RctPtInRectangle(pScreenX,pScreenY,&StatusBar.RectStat[4]))
			return keycode;
		else if (keycode == vchrCalc && (KeyStruct[ConfigReset].Key) &&  (PalmPollKeyKey(KeyStruct[ConfigReset].Key,CurrentKey)))
			  return keycode;
		else if ((keycode == vchrCalc) && MenuOption > 0 && pPenDown && RctPtInRectangle(pScreenX,pScreenY,&StatusBar.RectStat[1]))
			return keycode;
		else if (keycode == 'A' && pPenDown)
		{
		  	if (RctPtInRectangle(pScreenX,pScreenY,&srcRectA))
		  		return keycode;
		}
		else if (keycode == 'B' && pPenDown)
		{
		  	if (RctPtInRectangle(pScreenX,pScreenY,&srcRectB))
		  		return keycode;
		}
		else if (keycode == 'C' && pPenDown)
		{
			if (RctPtInRectangle(pScreenX,pScreenY,&srcRectC))
				return keycode;
		}
		else if (keycode == 'D' && pPenDown)
		{
			if (RctPtInRectangle(pScreenX,pScreenY,&srcRectD))
				return keycode;
		}
		else if (keycode == 'E' && pPenDown)
		{
			if (RctPtInRectangle(pScreenX,pScreenY,&srcRectE))
		  		return keycode;
		}
		else
			return 0;

}

/*
  wait for the user to press a key and return its code. This function is not
  required to do anything, it is here so we can avoid bogging down multitasking
  systems while using the debugger. If you don't want to or can't support this
  function you can just return OSD_KEY_NONE.
*/
int Keyboard_wait_keypress(void)
{
    
    return OSD_KEY_NONE;
}

void Keyboard_CustomizeInputportDefaults(int DefaultInput, struct ipd *defaults)
{
	int j;

	if (DefaultInput == INPUT_LAYOUT_HR
    ||  DefaultInput == INPUT_LAYOUT_HRSE)
	{
		while (defaults->type != IPT_END)
		{
			if (defaults->type == IPT_UI_TOGGLE_UI)                        seq_set_1(&defaults->seq, KEYCODE_TAB);
			if (defaults->type == IPT_UI_SELECT)                           seq_set_1(&defaults->seq, KEYCODE_ENTER);
			if (defaults->type == IPT_UI_CANCEL)                           seq_set_1(&defaults->seq, KEYCODE_ESC);
			if (defaults->type == IPT_START1)                              seq_set_1(&defaults->seq, KEYCODE_1);
			if (defaults->type == IPT_START2)                              seq_set_1(&defaults->seq, KEYCODE_2);
			if (defaults->type == IPT_COIN1)                               seq_set_1(&defaults->seq, KEYCODE_5);
			if (defaults->type == IPT_COIN2)                               seq_set_1(&defaults->seq, KEYCODE_6);
			if (defaults->type == (IPT_JOYSTICKRIGHT_UP    | IPF_PLAYER1)) seq_set_1(&defaults->seq, KEYCODE_SPACE);
			if (defaults->type == (IPT_JOYSTICKRIGHT_DOWN  | IPF_PLAYER1)) seq_set_1(&defaults->seq, KEYCODE_LCONTROL);
			if (defaults->type == (IPT_JOYSTICKRIGHT_LEFT  | IPF_PLAYER1)) seq_set_1(&defaults->seq, KEYCODE_LSHIFT);
			if (defaults->type == (IPT_JOYSTICKRIGHT_RIGHT | IPF_PLAYER1)) seq_set_1(&defaults->seq, KEYCODE_LALT);
			if (defaults->type == (IPT_JOYSTICKLEFT_UP     | IPF_PLAYER1)) seq_set_1(&defaults->seq, KEYCODE_UP);
			if (defaults->type == (IPT_JOYSTICKLEFT_DOWN   | IPF_PLAYER1)) seq_set_1(&defaults->seq, KEYCODE_DOWN);
			if (defaults->type == (IPT_JOYSTICKLEFT_LEFT   | IPF_PLAYER1)) seq_set_1(&defaults->seq, KEYCODE_LEFT);
			if (defaults->type == (IPT_JOYSTICKLEFT_RIGHT  | IPF_PLAYER1)) seq_set_1(&defaults->seq, KEYCODE_RIGHT);
			if (defaults->type == (IPT_BUTTON1 | IPF_PLAYER1))             seq_set_1(&defaults->seq, KEYCODE_LCONTROL);
			if (defaults->type == (IPT_BUTTON2 | IPF_PLAYER1))             seq_set_1(&defaults->seq, KEYCODE_LALT);
			if (defaults->type == (IPT_BUTTON3 | IPF_PLAYER1))             seq_set_1(&defaults->seq, KEYCODE_SPACE);
			if (defaults->type == (IPT_BUTTON4 | IPF_PLAYER1))             seq_set_1(&defaults->seq, KEYCODE_LSHIFT);
//			if (defaults->type == (IPT_BUTTON5 | IPF_PLAYER1))             seq_set_1(&defaults->seq, CODE_NONE);
//			if (defaults->type == (IPT_BUTTON6 | IPF_PLAYER1))             seq_set_1(&defaults->seq, CODE_NONE);
			if (defaults->type == (IPT_BUTTON1 | IPF_PLAYER2))             seq_set_1(&defaults->seq, KEYCODE_LCONTROL);
			if (defaults->type == (IPT_BUTTON2 | IPF_PLAYER2))             seq_set_1(&defaults->seq, KEYCODE_LALT);
			if (defaults->type == (IPT_BUTTON3 | IPF_PLAYER2))             seq_set_1(&defaults->seq, KEYCODE_SPACE);
			if (defaults->type == (IPT_BUTTON4 | IPF_PLAYER2))             seq_set_1(&defaults->seq, KEYCODE_LSHIFT);
//			if (defaults->type == (IPT_BUTTON5 | IPF_PLAYER2))             seq_set_1(&defaults->seq, CODE_NONE);
//			if (defaults->type == (IPT_BUTTON6 | IPF_PLAYER2))             seq_set_1(&defaults->seq, CODE_NONE);
			if (defaults->type == (IPT_BUTTON3 | IPF_PLAYER3))             seq_set_1(&defaults->seq, CODE_NONE);

			for (j = 0; j < SEQ_MAX; ++j)
			{
				if (defaults->seq[j] == KEYCODE_UP)    defaults->seq[j] = KEYCODE_UP;
				if (defaults->seq[j] == KEYCODE_DOWN)  defaults->seq[j] = KEYCODE_DOWN;
				if (defaults->seq[j] == KEYCODE_LEFT)  defaults->seq[j] = KEYCODE_LEFT;
				if (defaults->seq[j] == KEYCODE_RIGHT) defaults->seq[j] = KEYCODE_RIGHT;
			}
			//if (defaults->type == IPT_UI_SELECT)                           seq_set_1(&defaults->seq, KEYCODE_C);
			//if (defaults->type == IPT_UI_TOGGLE_UI)                           seq_set_1(&defaults->seq, KEYCODE_D);
			//if (defaults->type == IPT_UI_CANCEL)                           seq_set_1(&defaults->seq, KEYCODE_ESC);
			if (defaults->type == IPT_UI_UP)                           seq_set_1(&defaults->seq, KEYCODE_UP);
			if (defaults->type == IPT_UI_DOWN)                           seq_set_1(&defaults->seq, KEYCODE_DOWN);
			if (defaults->type == IPT_UI_LEFT)                           seq_set_1(&defaults->seq, KEYCODE_LEFT);
			if (defaults->type == IPT_UI_RIGHT)                           seq_set_1(&defaults->seq, KEYCODE_RIGHT);
			
			//if (defaults->type == IPT_START1)                              seq_set_1(&defaults->seq, '1');
			//if (defaults->type == IPT_START2)                              seq_set_1(&defaults->seq, '2');
			//if (defaults->type == IPT_COIN1)                               seq_set_1(&defaults->seq, '5');
			//if (defaults->type == IPT_COIN2)                               seq_set_1(&defaults->seq, '6');

			if (defaults->type == (IPT_BUTTON1 | IPF_PLAYER1))             seq_set_1(&defaults->seq, KEYCODE_LCONTROL);
			if (defaults->type == (IPT_BUTTON2 | IPF_PLAYER1))             seq_set_1(&defaults->seq, KEYCODE_LALT);
			if (defaults->type == (IPT_BUTTON3 | IPF_PLAYER1))             seq_set_1(&defaults->seq, KEYCODE_SPACE);
			if (defaults->type == (IPT_BUTTON4 | IPF_PLAYER1))             seq_set_1(&defaults->seq, KEYCODE_LSHIFT);
			if (defaults->type == (IPT_BUTTON5 | IPF_PLAYER1))             seq_set_1(&defaults->seq, CODE_NONE);
			if (defaults->type == (IPT_BUTTON6 | IPF_PLAYER1))             seq_set_1(&defaults->seq, CODE_NONE);
			if (defaults->type == (IPT_BUTTON1 | IPF_PLAYER2))             seq_set_1(&defaults->seq, KEYCODE_LCONTROL);
			if (defaults->type == (IPT_BUTTON2 | IPF_PLAYER2))             seq_set_1(&defaults->seq, KEYCODE_LALT);
			if (defaults->type == (IPT_BUTTON3 | IPF_PLAYER2))             seq_set_1(&defaults->seq, KEYCODE_SPACE);
			if (defaults->type == (IPT_BUTTON4 | IPF_PLAYER2))             seq_set_1(&defaults->seq, KEYCODE_LSHIFT);
			if (defaults->type == (IPT_BUTTON5 | IPF_PLAYER2))             seq_set_1(&defaults->seq, CODE_NONE);
			if (defaults->type == (IPT_BUTTON6 | IPF_PLAYER2))             seq_set_1(&defaults->seq, CODE_NONE);
			if (defaults->type == (IPT_UI_RESET_MACHINE))					seq_set_1(&defaults->seq, KEYCODE_F3);
			
			defaults++;
		}
	}
}

/***************************************************************************
    Message handlers
 ***************************************************************************/

/*static BOOL Keyboard_OnMessage(HWND hWnd, UINT Msg, WPARAM wParam, LPARAM lParam, LRESULT* pResult)
{
    return FALSE;
}*/

