#include "C68000.h"
#include "driver.h"
#include "cpuintrf.h"
//#include <stdio.h>

int cyclone_ICount;
int pending_interrupts;

struct Cyclone cyclone;
unsigned char *cyclone_ram;
unsigned char *cyclone_rom;
static int previous_pc;


static unsigned char MyRead8(unsigned int a) {
	return cpu_readmem24((a)&0xffffff);	
}

static unsigned short MyRead16(unsigned int a) {
	return cpu_readmem24_word((a)&0xffffff);
}

static unsigned int MyRead32(unsigned int a) {
	return cpu_readmem24_dword((a)&0xffffff);
}

static void MyWrite8(unsigned int a,unsigned char d) {
	cpu_writemem24((a)&0xffffff,d);
}

static void MyWrite16(unsigned int a,unsigned short d) {
	cpu_writemem24_word((a)&0xffffff,d);
}

static void MyWrite32(unsigned int a,unsigned int d) {
	cpu_writemem24_dword((a)&0xffffff,d);
}

static unsigned int MyCheckPc(unsigned int pc)
{
  	pc = (pc-cyclone.membase) & 0xffffff; /* Get the real program counter */
	change_pc24(pc);
	cyclone.membase=(unsigned int)OP_ROM;
  	return cyclone.membase+pc; /* New program counter */
}

static int MyIrqCallback(int irq)
{
	cyclone.irq=0;
	if(pending_interrupts & 0xff000000)
	{
		int level, mask = 0x80000000;
		for (level = 7; level; level--, mask >>= 1)
			if (pending_interrupts & mask)
				break;
   		if(level>cyclone.irq)
   		{
   			pending_interrupts &= ~(1 << (level + 24));
   			cyclone.irq = level;
   		}
	}

	return cyclone.irq;	
}

void cyclone_Reset(void)
{
	memset(&cyclone, 0,sizeof(cyclone));
	cyclone.checkpc=MyCheckPc;
	cyclone.read8  =MyRead8;
	cyclone.read16 =MyRead16;
	cyclone.read32 =MyRead32;
	cyclone.write8 =MyWrite8;
	cyclone.write16=MyWrite16;
	cyclone.write32=MyWrite32;
	cyclone.fetch8  =MyRead8;
	cyclone.fetch16 =MyRead16;
	cyclone.fetch32 =MyRead32;
	cyclone.IrqCallback=MyIrqCallback; /* 0 */
  	cyclone.srh=0x27; /* Set supervisor mode */
  	cyclone.a[7]=cyclone.read32(0); /* Get Stack Pointer */
  	cyclone.membase=0;

	{  	
  		extern unsigned char *OP_RAM;
		extern unsigned char *OP_ROM;
		cyclone_ram=OP_RAM;
		cyclone_rom=OP_ROM;
	}
		__asm 
    	{
    		LDR 	R0, = cyclone
	  	    ADD		R0,R0,R10
  			STR	 	R9,[R0,#0x8C]
  			STR	 	R10,[R0,#0x90]
	    } 

  	cyclone.pc=cyclone.checkpc(cyclone.read32(4)); /* Get Program Counter */

   	pending_interrupts = 0;
}

int  cyclone_GetPC(void)
{
	return (cyclone.pc - cyclone.membase);
}

void cyclone_SetRegs(Cyclone_Regs *src)
{
	memcpy(&cyclone,&(src->regs),sizeof(cyclone));
	pending_interrupts = src->pending_interrupts;
}

unsigned cyclone_GetRegs(Cyclone_Regs *dst)
{
	if( dst )
	{
		memcpy(&(dst->regs),&cyclone,sizeof(cyclone));
		dst->pending_interrupts = pending_interrupts;
	}
	return sizeof(Cyclone_Regs);

}

void cyclone_Cause_Interrupt(int level)
{
	if (level >= 1 && level <= 7) {
		pending_interrupts |= 1 << (24 + level);
	}
}

void cyclone_Clear_Pending_Interrupts(void)
{
	pending_interrupts &= ~0xff000000;
}

int cyclone_Execute(int cycles)
{
	if ((pending_interrupts & cyclone_STOP) && !(pending_interrupts & 0xff000000))
		return cycles;

	if (pending_interrupts & 0xff000000) 
	{
		int level, mask = 0x80000000;
		for (level = 7; level; level--, mask >>= 1)
			if (pending_interrupts & mask)
				break;
		if(level>cyclone.irq)
		{
			pending_interrupts &= ~(1 << (level + 24));
			cyclone.irq = level;
		}
	}

	cyclone.cycles = cycles;
	{
		extern int previous_pc;
		previous_pc = (cyclone.pc - cyclone.membase) & 0xffffff;
	}
	CycloneRun(&cyclone);
   	return (cycles - cyclone.cycles);
}

void cyclone_exit(void)
{}

void cyclone_set_pc(unsigned val)
{
	//cyclone.pc = val;
	//MyCheckPc(val);
}
unsigned cyclone_get_sp(void)
{
	//return cyclone.osp;
}

void cyclone_set_sp(unsigned val)
{
	//cyclone.osp = val;
}
unsigned cyclone_get_reg(int regnum)
{
    /*switch( regnum )
    {
		case M68K_PC: return cyclone.pc;
		case M68K_ISP: return cyclone.osp;
		//case M68K_USP: return cyclone.usp;
		case M68K_SR: return cyclone.srh;
		//case M68K_VBR: return cyclone.vbr;
		//case M68K_SFC: return cyclone.sfc;
		//case M68K_DFC: return cyclone.dfc;
		case M68K_D0: return cyclone.d[0];
		case M68K_D1: return cyclone.d[1];
		case M68K_D2: return cyclone.d[2];
		case M68K_D3: return cyclone.d[3];
		case M68K_D4: return cyclone.d[4];
		case M68K_D5: return cyclone.d[5];
		case M68K_D6: return cyclone.d[6];
		case M68K_D7: return cyclone.d[7];
		case M68K_A0: return cyclone.a[0];
		case M68K_A1: return cyclone.a[1];
		case M68K_A2: return cyclone.a[2];
		case M68K_A3: return cyclone.a[3];
		case M68K_A4: return cyclone.a[4];
		case M68K_A5: return cyclone.a[5];
		case M68K_A6: return cyclone.a[6];
		case M68K_A7: return cyclone.a[7];
		case REG_PREVIOUSPC: return previous_pc;
// TODO: Verify that this is the right thing to do for the purpose? 
		default:
			if( regnum <= REG_SP_CONTENTS )
			{
				unsigned offset = cyclone.osp + 4 * (REG_SP_CONTENTS - regnum);
				if( offset < 0xfffffd )
					return cpu_readmem24_dword( offset );
            }
    }*/
    return 0;
}

void cyclone_set_reg(int regnum, unsigned val)
{
    /*switch( regnum )
    {
		case M68K_PC: cyclone.pc = val; break;
		case M68K_ISP: cyclone.osp = val; break;
		//case M68K_USP: cyclone.usp = val; break;
		case M68K_SR: cyclone.srh = val; break;
		//case M68K_VBR: cyclone.vbr = val; break;
		//case M68K_SFC: cyclone.sfc = val; break;
		//case M68K_DFC: cyclone.dfc = val; break;
		case M68K_D0: cyclone.d[0] = val; break;
		case M68K_D1: cyclone.d[1] = val; break;
		case M68K_D2: cyclone.d[2] = val; break;
		case M68K_D3: cyclone.d[3] = val; break;
		case M68K_D4: cyclone.d[4] = val; break;
		case M68K_D5: cyclone.d[5] = val; break;
		case M68K_D6: cyclone.d[6] = val; break;
		case M68K_D7: cyclone.d[7] = val; break;
		case M68K_A0: cyclone.a[0] = val; break;
		case M68K_A1: cyclone.a[1] = val; break;
		case M68K_A2: cyclone.a[2] = val; break;
		case M68K_A3: cyclone.a[3] = val; break;
		case M68K_A4: cyclone.a[4] = val; break;
		case M68K_A5: cyclone.a[5] = val; break;
		case M68K_A6: cyclone.a[6] = val; break;
		case M68K_A7: cyclone.a[7] = val; break;
// TODO: Verify that this is the right thing to do for the purpose? 
		default:
			if( regnum <= REG_SP_CONTENTS )
			{
				unsigned offset = cyclone.osp + 4 * (REG_SP_CONTENTS - regnum);
				if( offset < 0xfffffd )
					cpu_writemem24_dword( offset, val );
            }
    }*/
}
void cyclone_set_nmi_line(int state)
{
	switch(state)
	{
		case CLEAR_LINE:
			cyclone_Clear_Pending_Interrupts();
			return;
		case ASSERT_LINE:
			cyclone_Cause_Interrupt(7);
			return;
		default:
			cyclone_Cause_Interrupt(7);
			return;
	}
}

void cyclone_set_irq_line(int irqline, int state)
{
	switch(state)
	{
		case CLEAR_LINE:
			cyclone_Clear_Pending_Interrupts();
			return;
		case ASSERT_LINE:
			cyclone_Cause_Interrupt(irqline);
			return;
		default:
			cyclone_Cause_Interrupt(irqline);
			return;
	}
}

void cyclone_set_irq_callback(int (*callback)(int irqline))
{
	//cyclone.IrqCallback = MyIrqCallback;
	cyclone.IrqCallback = MyIrqCallback;
}

/****************************************************************************
 * Return a formatted string for a register
 ****************************************************************************/
const char *cyclone_info(void *context, int regnum)
{
    static char buffer[32][47+1];
	static int which;
	//cyclone *r = context;

	which = ++which % 32;
	buffer[which][0] = '\0';
	//if( !context )
	//	r = &cyclone;

	switch( regnum )
	{
		/*case CPU_INFO_REG+M68K_PC: sprintf(buffer[which], "PC:%06X", r->pc); break;
		case CPU_INFO_REG+M68K_ISP: sprintf(buffer[which], "ISP:%08X", r->isp); break;
		case CPU_INFO_REG+M68K_USP: sprintf(buffer[which], "USP:%08X", r->usp); break;
		case CPU_INFO_REG+M68K_SR: sprintf(buffer[which], "SR:%08X", r->sr); break;
		case CPU_INFO_REG+M68K_VBR: sprintf(buffer[which], "VBR:%08X", r->vbr); break;
		case CPU_INFO_REG+M68K_SFC: sprintf(buffer[which], "SFC:%08X", r->sfc); break;
		case CPU_INFO_REG+M68K_DFC: sprintf(buffer[which], "DFC:%08X", r->dfc); break;
		case CPU_INFO_REG+M68K_D0: sprintf(buffer[which], "D0:%08X", r->d[0]); break;
		case CPU_INFO_REG+M68K_D1: sprintf(buffer[which], "D1:%08X", r->d[1]); break;
		case CPU_INFO_REG+M68K_D2: sprintf(buffer[which], "D2:%08X", r->d[2]); break;
		case CPU_INFO_REG+M68K_D3: sprintf(buffer[which], "D3:%08X", r->d[3]); break;
		case CPU_INFO_REG+M68K_D4: sprintf(buffer[which], "D4:%08X", r->d[4]); break;
		case CPU_INFO_REG+M68K_D5: sprintf(buffer[which], "D5:%08X", r->d[5]); break;
		case CPU_INFO_REG+M68K_D6: sprintf(buffer[which], "D6:%08X", r->d[6]); break;
		case CPU_INFO_REG+M68K_D7: sprintf(buffer[which], "D7:%08X", r->d[7]); break;
		case CPU_INFO_REG+M68K_A0: sprintf(buffer[which], "A0:%08X", r->a[0]); break;
		case CPU_INFO_REG+M68K_A1: sprintf(buffer[which], "A1:%08X", r->a[1]); break;
		case CPU_INFO_REG+M68K_A2: sprintf(buffer[which], "A2:%08X", r->a[2]); break;
		case CPU_INFO_REG+M68K_A3: sprintf(buffer[which], "A3:%08X", r->a[3]); break;
		case CPU_INFO_REG+M68K_A4: sprintf(buffer[which], "A4:%08X", r->a[4]); break;
		case CPU_INFO_REG+M68K_A5: sprintf(buffer[which], "A5:%08X", r->a[5]); break;
		case CPU_INFO_REG+M68K_A6: sprintf(buffer[which], "A6:%08X", r->a[6]); break;
		case CPU_INFO_REG+M68K_A7: sprintf(buffer[which], "A7:%08X", r->a[7]); break;
		case CPU_INFO_FLAGS:
			sprintf(buffer[which], "%c%c%c%c%c%c%c%c%c%c%c%c%c%c%c%c",
				r->sr & 0x8000 ? 'T':'.',
				r->sr & 0x4000 ? '?':'.',
				r->sr & 0x2000 ? 'S':'.',
				r->sr & 0x1000 ? '?':'.',
				r->sr & 0x0800 ? '?':'.',
				r->sr & 0x0400 ? 'I':'.',
				r->sr & 0x0200 ? 'I':'.',
				r->sr & 0x0100 ? 'I':'.',
				r->sr & 0x0080 ? '?':'.',
				r->sr & 0x0040 ? '?':'.',
				r->sr & 0x0020 ? '?':'.',
				r->sr & 0x0010 ? 'X':'.',
				r->sr & 0x0008 ? 'N':'.',
				r->sr & 0x0004 ? 'Z':'.',
				r->sr & 0x0002 ? 'V':'.',
				r->sr & 0x0001 ? 'C':'.');
            break;*/
		case CPU_INFO_NAME: return "Cyclone 68000";
		case CPU_INFO_FAMILY: return "Motorola 68K";
		case CPU_INFO_VERSION: return "0.16";
		case CPU_INFO_FILE: return __FILE__;
		case CPU_INFO_CREDITS: return "Copyright 1998,99 Mike Coates, Darren Olafson. All rights reserved";
		//case CPU_INFO_REG_LAYOUT: return (const char*)m68k_reg_layout;
       // case CPU_INFO_WIN_LAYOUT: return (const char*)m68k_win_layout;
	}
	return buffer[which];
}

unsigned cyclone_dasm(char *buffer, unsigned pc)
{
	change_pc24(pc);
#ifdef MAME_DEBUG
    return m68k_disassemble(buffer, pc);
#else
	sprintf(buffer, "$%04X", cpu_readop16(pc) );
	return 2;
#endif
}
