/*
	File:			utils.c

	Description:	Utility Functions

	Author:		

	Copyright:		Copyright  2003 by Tapwave, Inc.

	Disclaimer:		IMPORTANT:  This Tapwave software is provided by Tapwave, Inc. ("Tapwave").  Your 
					use is subject to and governed by terms and conditions of the Software Development 
					Kit Agreement ("SDK Agreement") between you and Tapwave.  If you have not entered 
					into Tapwaves standard SDK Agreement with Tapwave, you have no right or license 
					to use, reproduce, modify, distribute or otherwise exploit this Tapwave software.  
					You may obtain a copy of Tapwaves standard SDK Agreement by calling 650-960-1817
					or visiting Tapwave at http://www.tapwave.com/developers/. 

	Change History (most recent first):
				
*/

#include <PalmOS.h>
#include "HostPalm.h"
#include "utils.h"
#include <stdio.h>
//#include <sys/stat.h>
#include <zlib.h>
#include <errno.h>
#include <fcntl.h>
#include <stdlib.h>
#include <string.h>

extern int iCutSceneCount;

#ifdef __cplusplus
extern "C" {
#endif


static UInt32 start_clock_count;


#if CPU_TYPE == CPU_68K
#define LittleEndianSwap16(x) \
	((((x) >> 8) & 0xFF) | \
	 (((x) & 0xFF) << 8))
#else
#define LittleEndianSwap16(x) \
    (x)
#endif

typedef struct 
{
	UInt16   signature;
	UInt16   _pad_;
	UInt32   size;
	UInt32   reserved;
	UInt32   dataOffset;
} BitmapFileHeader;

typedef struct 
{
	UInt32	sizeInfoHeader;
	UInt32	width;
	UInt32	height;
	UInt16	planes;
	UInt16	bitCount;
	UInt32	compression;
	UInt32	sizeGlyph;
	UInt32	xPixelsPerM;
	UInt32	yPixelsPerM;
	UInt32	colorsUsed;
	UInt32	colorsImportant;
} BitmapInfoHeader;

#if defined(__arm) || defined(_MSC_VER)
#define ENDIAN '<'
#else
#define ENDIAN '>'
#endif

const char*
pack(const char* format, char* buf, const void* source)
{
    int bswap = 0;
    const char* src = (char*) source;

    for (;;) {
        char ch = *format++;
        switch (ch) {
        case 0:
            return src;
        case '<': case '>':
            bswap = (ch != ENDIAN);
            continue;
        case '=':
            bswap = 0;
            continue;
        case '!':
            bswap = 1;
            continue;
        case 'b': case 'B':
            *buf++ = *src++;
            continue;
        case 'h': case 'H':
            if (bswap) {
                buf[1] = src[0];
                buf[0] = src[1];
            } else {
                buf[0] = src[0];
                buf[1] = src[1];
            }
            buf += 2;
            src += 2;
            continue;
        case 'i': case 'I': case 'f':
            if (bswap) {
                buf[3] = src[0];
                buf[2] = src[1];
                buf[1] = src[2];
                buf[0] = src[3];
            } else {
                buf[0] = src[0];
                buf[1] = src[1];
                buf[2] = src[2];
                buf[3] = src[3];
            }
            buf += 4;
            src += 4;
            continue;
        case 'l': case 'L': case 'd':
            if (bswap) {
                buf[7] = src[0];
                buf[6] = src[1];
                buf[5] = src[2];
                buf[4] = src[3];
                buf[3] = src[4];
                buf[2] = src[5];
                buf[1] = src[6];
                buf[0] = src[7];
            } else {
                buf[0] = src[0];
                buf[1] = src[1];
                buf[2] = src[2];
                buf[3] = src[3];
                buf[0] = src[4];
                buf[1] = src[5];
                buf[2] = src[6];
                buf[3] = src[7];
            }
            buf += 4;
            src += 4;
            continue;
        case 'x':
            buf += 1;
            continue;
        case 'X':
            buf -= 1;
            continue;
        case 'y':
            src += 1;
            continue;
        case 'Y':
            src -= 1;
            continue;
		case ' ': case ',':
			continue;
        default:
            DbgBreak();
            continue;
        }
    }

	return src;
}


char * LoadImage(char *filename)
{
	FILE *fp;
	char *dataP;
	int size=0;
	
    size = file_size(filename);
	if((fp=fopen(filename,"rb"))==NULL)
    	return NULL;
    	
    dataP = (char *)malloc(size);
    fread(dataP,1,size,fp);
    fclose(fp);
    	
	return dataP;
}



Err
AppReadBMP(HBITMAP *bmp, const void * dataP)
{
    int compression;
    int row;
    int column;
    Int32 sourceBytesPerRow;
    Err error=0;
	BitmapFileHeader fileHeader;
	BitmapInfoHeader infoHeader;
	const void * p = dataP;
	const void * newDataP = NULL;
	int red;
	int green;
	int blue;
    
	// unpack the fileHeader structure.  This is to solve alignment differences on
	// 68K, x86, and ARM.
	p = pack("<hxxiii", (char*)&fileHeader, (const char*)p);
	pack("<iiihhiiiiii", (char*)&infoHeader, (const char*)p);
    
	if (fileHeader.signature != 'MB' && fileHeader.signature != 'BM') return 1; // 'BM' for 68K
	
    bmp->bmWidth = (Int32) infoHeader.width;
    bmp->bmHeight = (Int32) infoHeader.height;
    
    compression = (int) infoHeader.compression;
    
    sourceBytesPerRow = (((infoHeader.bitCount * bmp->bmWidth + 7) / 8 + 3) / 4) * 4;
    
    switch (infoHeader.bitCount)
    {
        case 16:
            bmp->rowbytes = (UInt16 *)malloc(bmp->bmWidth * bmp->bmHeight * sizeof(UInt16));
            
            // 16 bit glyphs can not be compressed.
            if (compression == 0)
            {
                UInt8 *pixelsDestP;
                UInt8 *pixelsSourceP;
                UInt16 *destP;
                UInt16 *sourceP;
                
                // The data must be converted to 565 RGB.
                pixelsDestP = (UInt8 *) MemGluePtrNewL((UInt32) (bmp->bmWidth * bmp->bmHeight * sizeof(UInt16)));
                pixelsSourceP = (UInt8 *) dataP + fileHeader.dataOffset;
                
                for (row = 0; row < bmp->bmHeight; row++)
                {
                    destP = (UInt16 *) (pixelsDestP + row * bmp->bmWidth * sizeof(UInt16));
                    sourceP = (UInt16 *) (pixelsSourceP + 
                        ((bmp->bmHeight - 1) - row) * sourceBytesPerRow);
                    
                    for (column = 0; column < bmp->bmWidth; column++)
                    {
                        // Convert from 555 to 565.
                        *destP = *sourceP & 31 | ((*sourceP & (0x7fff ^ 31)) << 1);
                        destP++;
                        sourceP++;
                    }
                }
                
                // The data may really be padded out to 32 bit boundaries.
                //error = TwGfxWriteSurface(*aResult, pixelsDestP, false);
                MemMove(bmp->rowbytes, pixelsDestP, bmp->bmWidth * bmp->bmHeight * sizeof(UInt16));
                free(pixelsDestP);
            }
            else if (compression == 3)
            {
                // DOLATER
                // Convert RGB from the bitmasks in RGBQUAD
            }
            break;
            
        case 24:
            bmp->rowbytes = (UInt16 *)malloc(bmp->bmWidth * bmp->bmHeight * sizeof(UInt16));
			            
            // 24 bit glyphs can not be compressed.
            if (compression == 0)
            {
                UInt8 *pixelsDestP;
                UInt8 *pixelsSourceP;
                UInt16 *destP;
                UInt8 *sourceP;
                UInt16 pixel;
                
                // The data must be converted to 565 RGB.
                pixelsDestP = (UInt8 *) MemGluePtrNewL((UInt32) (bmp->bmWidth * bmp->bmHeight * sizeof(UInt16)));
                pixelsSourceP = (UInt8 *) dataP + fileHeader.dataOffset;
                
                for (row = 0; row < bmp->bmHeight; row++)
                {
                    destP = (UInt16 *) (pixelsDestP + row * bmp->bmWidth * sizeof(UInt16));
                    sourceP = pixelsSourceP + ((bmp->bmHeight - 1) - row) * sourceBytesPerRow;
                    
                    for (column = 0; column < bmp->bmWidth; column++)
                    {
                        // Convert from BGR 888 to RGB 565.
#if 1
                        blue = *sourceP++;
                        green = *sourceP++;
                        red = *sourceP++;
                        pixel = TwGfxMakeDisplayRGB(red, green, blue);
#else
                        pixel = (UInt16) (*sourceP++ >> 3);
                        pixel |= (UInt16) ((*sourceP++ >> 2) << 5);
                        pixel |= (UInt16) ((*sourceP++ >> 3) << 11);
#endif
                        pixel = LittleEndianSwap16(pixel);
                        *destP++ = pixel;
                    }
                }
                
                // The data may really be padded out to 32 bit boundaries.
                //error = TwGfxWriteSurface(*aResult, pixelsDestP, false);
                MemMove(bmp->rowbytes, pixelsDestP, bmp->bmWidth * bmp->bmHeight * sizeof(UInt16));
                free(pixelsDestP);
            }
            else if (compression == 3)
            {
                // DOLATER - 
                // Convert RGB from the bitmasks in RGBQUAD
            }
            break;
    }
    
    if (newDataP != NULL)
    	MemPtrFree((void *) newDataP);
        	
    return error;
    
}




#ifdef __cplusplus
} // extern "C"
#endif
